#!/usr/bin/python
# 
# BS Prefix Generator
# 
# This divides up a /8 network into a bunch of separate announcments of 
# more specific prefixes, and iterates with successively smaller subnets 
# to match the number of prefixes specified
#
# Blackhole Networks
#
# version = 0.1
#
# Define the /8 network to be split up with the variable ClassA 

ClassA = 13

import random, sys, argparse, math

parser = argparse.ArgumentParser(description='Create a bunch of network prefixes - up to 2^24.')
parser.add_argument('prefixes', type=int, help='print a bunch of network prefixes CIDR style from the IPv4 space')
args = parser.parse_args()

#prefixes = args.prefixes

def prefixdump( prefixes ):
   # This is <ClassA>.0.0.0 and <ClassA>.255.255.255 converted to decimal
   ip_low = ClassA*(2**24)
   ip_high = ip_low + 2**24 -1
   
   #This is the size of a /8 network in terms of addresses
   net_size = 2**(32-8)

   block_size = int ( round ( math.log ( (net_size/prefixes),2 ) ) ) 
   # And the netmask needed to obtain the appropriate number of prefixes convert to a CIDR style mask
   #cidr_mask = 32 -  ( int ( math.log ( (net_size/prefixes),2 ) ) ) 
   cidr_mask = 32 -  block_size

   # Convert our mask length into a 32 bit number
   submask = 0
   for i in range (cidr_mask):
      submask=submask+2**(31-i)

   for ip in range ( ip_low , ip_high , (2 ** ( 32 - cidr_mask )) ):
      network = ip & submask
      # print the network ID out in dotted quad notation 
      # do this by masking and bit shifting
      oct1 = (network & 0b11111111000000000000000000000000) >> 24
      oct2 = (network & 0b00000000111111110000000000000000) >> 16
      oct3 = (network & 0b00000000000000001111111100000000) >> 8
      oct4 = (network & 0b00000000000000000000000011111111) >> 0

      # Print out the network in CIDR notation
      sys.stdout.write("%s.%s.%s.%s/%s\n" % ( oct1, oct2, oct3, oct4, cidr_mask))
     
   return

# Main
prefixes = args.prefixes
# Do a quick sanity check
if ( prefixes > 2**24 ):
   sys.stderr.write('Can\'t fit that many prefixes in a /8 block. Try again with a value less than or equal to 16777216\n')
   exit(-1)

# Iterate through the number of prefixes using different network block sizes
while ( prefixes > 0 ):
   # Calculate our base CIDR mask size which is log base 2 of the number of prefixes were calling 
   # rounded down to the integer value
   pfx_len = int (math.log (prefixes,2))
   # The length of the network block well send to get printed is the CIDR mask to the power of 2
   pfx_len_blk = 2**pfx_len
   # Get the networks printed for the CIDR mask
   prefixdump ( pfx_len_blk )
   # The number of prefixes we have left is mod the block length
   prefixes = prefixes % pfx_len_blk
